/*
 * The OpenSAML License, Version 1.
 * Copyright (c) 2002
 * University Corporation for Advanced Internet Development, Inc.
 * All rights reserved
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution, if any, must include
 * the following acknowledgment: "This product includes software developed by
 * the University Corporation for Advanced Internet Development
 * <http://www.ucaid.edu>Internet2 Project. Alternately, this acknowledegement
 * may appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * Neither the name of OpenSAML nor the names of its contributors, nor
 * Internet2, nor the University Corporation for Advanced Internet Development,
 * Inc., nor UCAID may be used to endorse or promote products derived from this
 * software without specific prior written permission. For written permission,
 * please contact opensaml@opensaml.org
 *
 * Products derived from this software may not be called OpenSAML, Internet2,
 * UCAID, or the University Corporation for Advanced Internet Development, nor
 * may OpenSAML appear in their name, without prior written permission of the
 * University Corporation for Advanced Internet Development.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND WITH ALL FAULTS. ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT ARE DISCLAIMED AND THE ENTIRE RISK
 * OF SATISFACTORY QUALITY, PERFORMANCE, ACCURACY, AND EFFORT IS WITH LICENSEE.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER, CONTRIBUTORS OR THE UNIVERSITY
 * CORPORATION FOR ADVANCED INTERNET DEVELOPMENT, INC. BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
-----------------------------------------
file  :   SAMLAction.cpp
project :   Policy Server
date :   
author    :   andy
rights    :   (c) Parthenon Computing
notes  :    extension to openSAML C++ library (based on Java class)
--------------------------------------------
*/

#include "internal.h"

using namespace saml;
using namespace std;

SAMLAction::SAMLAction(const XMLCh* data, const XMLCh* nsURI) : m_namespace(NULL), m_data(NULL)
{
    RTTI(SAMLAction);
    if(!data || *data==chNull)
        throw MalformedException("SAMLAction() requires data");

    if(nsURI && *nsURI)
        m_namespace = XMLString::replicate(nsURI);

    m_data = XMLString::replicate(data);
}

SAMLAction::SAMLAction(DOMElement* e) : m_data(NULL), m_namespace(NULL)
{
    RTTI(SAMLAction);
    fromDOM(e);
}

SAMLAction::SAMLAction(std::istream& in) : SAMLObject(in), m_data(NULL), m_namespace(NULL)
{
    RTTI(SAMLAction);
    fromDOM(m_document->getDocumentElement());
}

SAMLAction::~SAMLAction()
{
    if (m_bOwnStrings) {
        delete[] const_cast<XMLCh*>(m_data);
        delete[] const_cast<XMLCh*>(m_namespace);
    }
}

void SAMLAction::checkValidity() const
{
    if (!m_data || *m_data==chNull)
        throw MalformedException("Action invalid, requires data");
}

SAMLObject* SAMLAction::clone() const
{
    return new SAMLAction(m_data, m_namespace);
}

void SAMLAction::fromDOM(DOMElement *e)
{
    SAMLObject::fromDOM(e);

    if (SAMLConfig::getConfig().strict_dom_checking &&
        (XMLString::compareString(XML::SAML_NS, e->getNamespaceURI()) || XMLString::compareString(L(Action), e->getLocalName())))
        throw MalformedException("SAMLAction() requires saml:Action at root");

    m_root = e;
    m_bOwnStrings = false;

    m_namespace = e->getAttributeNS(NULL, L(Namespace));
    m_data = e->getFirstChild()->getNodeValue();

    checkValidity();
}

DOMNode* SAMLAction::toDOM(DOMDocument* doc, bool xmlns) const
{
    SAMLObject::toDOM(doc, xmlns);
    if (m_root) {
        if (xmlns)
            static_cast<DOMElement*>(m_root)->setAttributeNS(XML::XMLNS_NS,L(xmlns),XML::SAML_NS);
        return m_root;
    }

    if (!doc)
        doc=m_document;

    // Construct an Action
    DOMElement* a=doc->createElementNS(XML::SAML_NS, L(Action));
    if (xmlns)
        a->setAttributeNS(XML::XMLNS_NS, L(xmlns), XML::SAML_NS);

    if(m_namespace && *m_namespace)
        a->setAttributeNS(NULL, L(Namespace), m_namespace);

    a->appendChild(doc->createTextNode(m_data));

    return m_root=a;
}

const XMLCh SAMLAction::SAML_ACTION_NAMESPACE_RWEDC_NEG[] = // "urn:oasis:names:tc:SAML:1.0:action:rwedc-negation"
{
    chLatin_u, chLatin_r, chLatin_n, chColon, chLatin_o, chLatin_a, chLatin_s, chLatin_i, chLatin_s, chColon,
    chLatin_n, chLatin_a, chLatin_m, chLatin_e, chLatin_s, chColon, chLatin_t, chLatin_c, chColon,
    chLatin_S, chLatin_A, chLatin_M, chLatin_L, chColon, chDigit_1, chPeriod, chDigit_0, chColon,
    chLatin_a, chLatin_c, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chColon,
    chLatin_r, chLatin_w, chLatin_e, chLatin_d, chLatin_c, chDash,
    chLatin_n, chLatin_e, chLatin_g, chLatin_a, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chNull
};

const XMLCh SAMLAction::SAML_ACTION_NAMESPACE_RWEDC[] = // "urn:oasis:names:tc:SAML:1.0:action:rwedc"
{
    chLatin_u, chLatin_r, chLatin_n, chColon, chLatin_o, chLatin_a, chLatin_s, chLatin_i, chLatin_s, chColon,
    chLatin_n, chLatin_a, chLatin_m, chLatin_e, chLatin_s, chColon, chLatin_t, chLatin_c, chColon,
    chLatin_S, chLatin_A, chLatin_M, chLatin_L, chColon, chDigit_1, chPeriod, chDigit_0, chColon,
    chLatin_a, chLatin_c, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chColon,
    chLatin_r, chLatin_w, chLatin_e, chLatin_d, chLatin_c, chNull
};

const XMLCh SAMLAction::SAML_ACTION_NAMESPACE_GHPP[] = // "urn:oasis:names:tc:SAML:1.0:action:ghpp"
{
    chLatin_u, chLatin_r, chLatin_n, chColon, chLatin_o, chLatin_a, chLatin_s, chLatin_i, chLatin_s, chColon,
    chLatin_n, chLatin_a, chLatin_m, chLatin_e, chLatin_s, chColon, chLatin_t, chLatin_c, chColon,
    chLatin_S, chLatin_A, chLatin_M, chLatin_L, chColon, chDigit_1, chPeriod, chDigit_0, chColon,
    chLatin_a, chLatin_c, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chColon,
    chLatin_g, chLatin_h, chLatin_p, chLatin_p, chNull
};

const XMLCh SAMLAction::SAML_ACTION_NAMESPACE_UNIX[] = // "urn:oasis:names:tc:SAML:1.0:action:unix"
{
    chLatin_u, chLatin_r, chLatin_n, chColon, chLatin_o, chLatin_a, chLatin_s, chLatin_i, chLatin_s, chColon,
    chLatin_n, chLatin_a, chLatin_m, chLatin_e, chLatin_s, chColon, chLatin_t, chLatin_c, chColon,
    chLatin_S, chLatin_A, chLatin_M, chLatin_L, chColon, chDigit_1, chPeriod, chDigit_0, chColon,
    chLatin_a, chLatin_c, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chColon,
    chLatin_u, chLatin_n, chLatin_i, chLatin_x, chNull
};
