/*
 * The OpenSAML License, Version 1.
 * Copyright (c) 2002
 * University Corporation for Advanced Internet Development, Inc.
 * All rights reserved
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution, if any, must include
 * the following acknowledgment: "This product includes software developed by
 * the University Corporation for Advanced Internet Development
 * <http://www.ucaid.edu>Internet2 Project. Alternately, this acknowledegement
 * may appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * Neither the name of OpenSAML nor the names of its contributors, nor
 * Internet2, nor the University Corporation for Advanced Internet Development,
 * Inc., nor UCAID may be used to endorse or promote products derived from this
 * software without specific prior written permission. For written permission,
 * please contact opensaml@opensaml.org
 *
 * Products derived from this software may not be called OpenSAML, Internet2,
 * UCAID, or the University Corporation for Advanced Internet Development, nor
 * may OpenSAML appear in their name, without prior written permission of the
 * University Corporation for Advanced Internet Development.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND WITH ALL FAULTS. ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT ARE DISCLAIMED AND THE ENTIRE RISK
 * OF SATISFACTORY QUALITY, PERFORMANCE, ACCURACY, AND EFFORT IS WITH LICENSEE.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER, CONTRIBUTORS OR THE UNIVERSITY
 * CORPORATION FOR ADVANCED INTERNET DEVELOPMENT, INC. BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* SAMLAuthenticationQuery.cpp - SAML authentication query implementation

   Scott Cantor
   5/7/04

   $History:$
*/

#include "internal.h"

using namespace saml;
using namespace std;

SAMLAuthenticationQuery::SAMLAuthenticationQuery(SAMLSubject* subject, const XMLCh* authMethod)
    : SAMLSubjectQuery(subject), m_authMethod(NULL)
{
    RTTI(SAMLAuthenticationQuery);
    m_authMethod = XMLString::replicate(authMethod);
}

SAMLAuthenticationQuery::SAMLAuthenticationQuery(DOMElement* e) : SAMLSubjectQuery(e), m_authMethod(NULL)
{
    RTTI(SAMLAuthenticationQuery);
    fromDOM(e);
}

SAMLAuthenticationQuery::SAMLAuthenticationQuery(std::istream& in) : SAMLSubjectQuery(in), m_authMethod(NULL)
{
    RTTI(SAMLAuthenticationQuery);
    fromDOM(m_document->getDocumentElement());
}

SAMLAuthenticationQuery::~SAMLAuthenticationQuery()
{
    if(m_bOwnStrings)
        XMLString::release(const_cast<XMLCh**>(&m_authMethod));
}

SAMLObject* SAMLAuthenticationQuery::clone() const
{
    return new SAMLAuthenticationQuery(static_cast<SAMLSubject*>(m_subject->clone()),m_authMethod);
}

void SAMLAuthenticationQuery::fromDOM(DOMElement* e)
{
    NDC ndc("fromDOM");

    if (SAMLConfig::getConfig().strict_dom_checking) {
        if (XMLString::compareString(XML::SAMLP_NS,e->getNamespaceURI()))
            throw MalformedException(SAMLException::REQUESTER,"SAMLAuthenticationQuery::fromDOM() root element isn't in samlp namespace");
        if (XMLString::compareString(L(AuthenticationQuery),e->getLocalName())) {
            auto_ptr<saml::QName> type(saml::QName::getQNameAttribute(e,XML::XSI_NS,L(type)));
            if ((XMLString::compareString(L(Query),e->getLocalName()) && XMLString::compareString(L(SubjectQuery),e->getLocalName())) ||
                !type.get() || XMLString::compareString(XML::SAMLP_NS,type->getNamespaceURI()) ||
                XMLString::compareString(L(AuthenticationQueryType),type->getLocalName()))
                throw MalformedException(SAMLException::REQUESTER,"SAMLAuthenticationQuery::fromDOM() missing samlp:AuthenticationQuery element at root");
        }
    }

    m_authMethod = e->getAttributeNS(NULL,L(AuthenticationMethod));
    checkValidity();
}

DOMNode* SAMLAuthenticationQuery::toDOM(DOMDocument* doc, bool xmlns) const
{
    SAMLObject::toDOM(doc, xmlns);
    if(m_root) {
        if(xmlns)
            static_cast<DOMElement*>(m_root)->setAttributeNS(XML::XMLNS_NS,L(xmlns),XML::SAMLP_NS);
        return m_root;
    }
    if (!doc)
        doc=m_document;

    // Construct an AuthenticationQuery
    DOMElement* q=doc->createElementNS(XML::SAMLP_NS,L(AuthenticationQuery));
    if (xmlns)
        q->setAttributeNS(XML::XMLNS_NS,L(xmlns),XML::SAMLP_NS);
    q->setAttributeNS(NULL,L(AuthenticationMethod),m_authMethod);
    q->appendChild(m_subject->toDOM(doc,true));

    return m_root=q;
}

