/*
 * The OpenSAML License, Version 1.
 * Copyright (c) 2002
 * University Corporation for Advanced Internet Development, Inc.
 * All rights reserved
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution, if any, must include
 * the following acknowledgment: "This product includes software developed by
 * the University Corporation for Advanced Internet Development
 * <http://www.ucaid.edu>Internet2 Project. Alternately, this acknowledegement
 * may appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * Neither the name of OpenSAML nor the names of its contributors, nor
 * Internet2, nor the University Corporation for Advanced Internet Development,
 * Inc., nor UCAID may be used to endorse or promote products derived from this
 * software without specific prior written permission. For written permission,
 * please contact opensaml@opensaml.org
 *
 * Products derived from this software may not be called OpenSAML, Internet2,
 * UCAID, or the University Corporation for Advanced Internet Development, nor
 * may OpenSAML appear in their name, without prior written permission of the
 * University Corporation for Advanced Internet Development.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND WITH ALL FAULTS. ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT ARE DISCLAIMED AND THE ENTIRE RISK
 * OF SATISFACTORY QUALITY, PERFORMANCE, ACCURACY, AND EFFORT IS WITH LICENSEE.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER, CONTRIBUTORS OR THE UNIVERSITY
 * CORPORATION FOR ADVANCED INTERNET DEVELOPMENT, INC. BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/* SAMLSubjectQuery.cpp - SAML subject query implementation

   Scott Cantor
   5/7/04

   $History:$
*/

#include "internal.h"

using namespace saml;
using namespace std;


SAMLSubjectQuery::SAMLSubjectQuery(SAMLSubject* subject) : m_subject(subject)
{
    RTTI(SAMLSubjectQuery);
}

SAMLSubjectQuery::SAMLSubjectQuery(DOMElement* e) : m_subject(NULL)
{
    RTTI(SAMLSubjectQuery);
    fromDOM(e);
}

SAMLSubjectQuery::SAMLSubjectQuery(istream& in) : SAMLQuery(in), m_subject(NULL)
{
    RTTI(SAMLSubjectQuery);
    fromDOM(m_document->getDocumentElement());
}

SAMLSubjectQuery::~SAMLSubjectQuery()
{
    delete m_subject;
}

void SAMLSubjectQuery::checkValidity() const
{
    if (!m_subject)
        throw MalformedException(SAMLException::RESPONDER, "SubjectStatement is invalid, requires subject");
}


SAMLObject* SAMLSubjectQuery::clone() const
{
    return new SAMLSubjectQuery(static_cast<SAMLSubject*>(m_subject->clone()));
}

void SAMLSubjectQuery::fromDOM(DOMElement* e)
{
    NDC ndc("fromDOM");
    SAMLObject::fromDOM(e);

    m_root=e;
    m_bOwnStrings=false;

    m_subject=new SAMLSubject(XML::getFirstChildElement(e));
}

void SAMLSubjectQuery::setSubject(SAMLSubject* subject)
{
    if (subject) {
        if (m_root)
            m_root->replaceChild(subject->toDOM(m_root->getOwnerDocument()), m_subject->toDOM());
        delete m_subject;
        m_subject = subject;
    }
    else
        throw MalformedException(SAMLException::RESPONDER, "subject cannot be null");
}
